#include <stdio.h>
#include <string.h>
#include <s1c33cpu.h>

#include "d12ci.h"
#include "mainloop.h"
#include "usb100.h"
#include "chap_9.h"

extern EPPFLAGS bEPPflags;
extern CONTROL_XFER ControlData;

/****************************************************************************
 *	connect
 ****************************************************************************/

/****************************************************************************
 *	ep2_txdone
 ****************************************************************************/


void main()
{
	BOOL in_loop = TRUE;
	unsigned char key, i;
#ifndef __C51__
	unsigned long ClockSec, last_clk;
#endif
	
	init_port();

	init_serial();

	init_timer0();

	init_special_interrupts();

	printf("\nPDIUSBD12 evaluation board firmware V2.2.\n");
	printf("Copyright (c) Philips Semiconductors, 1998.\n");

	/* Power on reset, lightup LEDs for 1 sec,
	   disconnect and reconnect Soft-Connect */
	printf("Re-connect PDIUSBD12 evaluation board to USB.\n");
	reconnect_USB();

	/* Main program loop */

	while( in_loop ){

#ifdef __C51__
		if (bEPPflags.bits.timer){
			DISABLE;
			bEPPflags.bits.timer = 0;
			ENABLE;

			if(bEPPflags.bits.configuration)
				check_key_LED();
		}

		if(RI) {
			key = _getkey();
#else
		if (bEPPflags.bits.timer){
			DISABLE;
			bEPPflags.bits.timer = 0;
			ENABLE;

			if(bEPPflags.bits.configuration)
				check_key_LED();

			ClockSec = ClockTicks*10/182;
			if(ClockSec != last_clk) {
				printf("Idle time = %ld sec.\r", (ClockTicks - ClockIsr)*10/182);
				last_clk = ClockSec;
			}
		}

		if( bioskey(1) ) {
			key = bioskey(0);
			key &= 0xff;
#endif
			switch(key) {
				case 0x1b: /* ESC */
#ifndef __C51__
					in_loop = FALSE;
#endif
					break;
				case 'r':
					D12_SendResume();
					break;
				case 'i':
					if(bEPPflags.bits.control_state == USB_IDLE)
						printf("Control state = USB_IDLE.");
					else if(bEPPflags.bits.control_state == USB_RECEIVE)
						printf("Control state = USB_RECEIVE.");
					else if(bEPPflags.bits.control_state == USB_TRANSMIT)
						printf("Control state = USB_TRANSMIT.");
					printf(" ControlData.wCount = %x.\n", ControlData.wCount);
#ifndef __C51__
					printf("Endpoint 4 (Bulk Out) Status = %x, Endpoint 5 (Bulk In) Status = %x.\n",
					   D12_ReadEndpointStatus(4), D12_ReadEndpointStatus(5));
					printf("ioSize = %x, ioCount = %x.\n", ioSize, ioCount);
#else
					printf("Endpoint 4 (Bulk Out) Status = %bx, Endpoint 5 (Bulk In) Status = %bx.\n",
					   D12_ReadEndpointStatus(4), D12_ReadEndpointStatus(5));
#endif
					if(bEPPflags.bits.dma_state == DMA_IDLE)
						printf("DMA_State = DMA_IDLE.\n");
					else if(bEPPflags.bits.dma_state == DMA_RUNNING)
						printf("DMA_State = DMA_RUNNING.\n");
					else if(bEPPflags.bits.dma_state == DMA_PENDING)
						printf("DMA_State = DMA_PENDING.\n");

#ifndef __C51__
					printf("Last Device Request: bmRequestType = 0x%x, bRequest= 0x%x, wValue = 0x%x, wLength = 0x%x, wIndex = 0x%x.\n",
						ControlData.DeviceRequest.bmRequestType,
						ControlData.DeviceRequest.bRequest,
						ControlData.DeviceRequest.wValue,
						ControlData.DeviceRequest.wLength,
						ControlData.DeviceRequest.wIndex);
#else
					MCU_HOSTDACK = 1;
					printf("CPLD counter read back = 0x%bx, 0x%bx.\n", inportb(CPLD_CNT0),
						inportb(CPLD_CNT1));
					MCU_HOSTDACK = 0;
					printf("Last Device Request: bmRequestType = 0x%bx, bRequest= 0x%bx, wValue = 0x%x, wLength = 0x%x, wIndex = 0x%x.\n",

						ControlData.DeviceRequest.bmRequestType,
						ControlData.DeviceRequest.bRequest,
						ControlData.DeviceRequest.wValue,
						ControlData.DeviceRequest.wLength,
						ControlData.DeviceRequest.wIndex);
#endif

					printf("Data: ");
					for(i = 0; i < ControlData.DeviceRequest.wLength; i ++)
#ifndef __C51__
						printf("0x%x, ", *((ControlData.dataBuffer)+i));
#else
						printf("0x%bx, ", *((ControlData.dataBuffer)+i));
#endif
					printf("\n");

					break;
				case '\r':
					reconnect_USB();
					break;

				case 'v':
					if(bEPPflags.bits.verbose == 0) {
						printf("Verbose Mode = ON.\n");
						DISABLE;
						bEPPflags.bits.verbose = 1;
						ENABLE;
					}
					else {
						printf("Verbose Mode = OFF.\n");
						DISABLE;
						bEPPflags.bits.verbose = 0;
						ENABLE;
					}
					break;
				default:
					break;
			}
		}


		if (bEPPflags.bits.bus_reset) {
			DISABLE;
			bEPPflags.bits.bus_reset = 0;
			ENABLE;
			printf("Bus reset!\n");
		} // if bus reset

		if (bEPPflags.bits.setup_packet){
			DISABLE;
			bEPPflags.bits.setup_packet = 0;
			ENABLE;
			control_handler();
		} // if setup_packet

	} // Main Loop

	on_exit();
}

void stall_ep0()
{
	D12_SetEndpointStatus(0, 1);
	D12_SetEndpointStatus(1, 1);
}

void init_unconfig()
{
	unsigned char i;

	D12_SetEndpointEnable(0);	/* Disable all endpoints but EPP0. */
}

void init_config()
{
	D12_SetEndpointEnable(1);	/* Enable  generic/iso endpoints. */
}

void single_transmit(unsigned char * buf, unsigned char len)
{
	if( len <= EP0_PACKET_SIZE) {
		D12_WriteEndpoint(1, buf, len);
	}
}

void code_transmit(unsigned char * pRomData, unsigned short len)
{
	ControlData.wCount = 0;
	if(ControlData.wLength > len)
		ControlData.wLength = len;

	ControlData.pData = pRomData;
	if( ControlData.wLength >= EP0_PACKET_SIZE) {
		D12_WriteEndpoint(1, ControlData.pData, EP0_PACKET_SIZE);
		ControlData.wCount += EP0_PACKET_SIZE;

		DISABLE;
		bEPPflags.bits.control_state = USB_TRANSMIT;
		ENABLE;
	}
	else {
		D12_WriteEndpoint(1, pRomData, ControlData.wLength);
		ControlData.wCount += ControlData.wLength;
		DISABLE;
		bEPPflags.bits.control_state = USB_IDLE;
		ENABLE;
	}
}

void control_handler()
{
	unsigned char type, req;

	type = ControlData.DeviceRequest.bmRequestType & USB_REQUEST_TYPE_MASK;
	req = ControlData.DeviceRequest.bRequest & USB_REQUEST_MASK;

	help_devreq(type, req); // print out device request

	if (type == USB_STANDARD_REQUEST)
		(*StandardDeviceRequest[req])();
	else if (type == USB_VENDOR_REQUEST)
		(*VendorDeviceRequest[req])();
	else
		stall_ep0();
}
